%
% Perform implicit Euler step.
% The finite difference scheme is written as
%
%  XN = FD(XO,L)
%  L  = Lambda(XN)
%
% Nonlinear solver of Matlab cannot solve the implicit
% nonlinear system XN = FC(X0,Lambda(XN)) so that an iterative procedure is
% used.
%
% X0 = old status X0(1) = x position
%                 X0(2) = y position
%                 X0(3) = x velcity
%                 X0(4) = y velcity
%
% DT = advancind time step
%
% omega, zeta = parameter for Baumgarte constraint stabilizartion
%
% m, g, r = parameter of the model, respectively mass, gravity an ray.
%
% Return XN the status of the model at t+DT.
%
function XN = EvalImplicitEulerStep( XO, DT, omega, zeta, m, g, r )
  % solve nonlinear system
  % EvalFD( NV, OV, DT, omega, zeta, g ) == 0
  XN = XO ;
  LN = EvalLambda( XN, omega, zeta, m, g, r ) ;
  for k=1:100
    LNold = LN ;
    XN    = EvalFD(XO, LN, DT, omega, zeta, m, g, r) ;
    LN    = EvalLambda( XN, omega, zeta, m, g, r );
    if abs(LN-LNold) < 1E-6 % if converged exit
      return ;
    end
  end
  %%
  error('EvalImplicitEulerStep, iterative procedure failed to converge') ;
end